package com.adc.da.util.persistence;

import com.adc.da.util.http.PageInfo;
import org.springframework.data.repository.NoRepositoryBean;
import org.springframework.data.repository.PagingAndSortingRepository;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

@NoRepositoryBean
public interface IBaseRepository<T, ID extends Serializable> extends PagingAndSortingRepository<T, ID> {

    /**
     * 保存Tree节点
     * @param entity
     * @return
     */
    T saveTreeEntity(T entity);

    /**
     * 逻辑删除
     * @param id
     */
    void deleteLogic(ID id);

    /**
     * 批量逻辑删除
     * @param ids
     */
    void deleteLogicInBatch(List<String> ids);

    /**
     * 删除Tree节点
     * @param id
     */
    void deleteTreeEntity(ID id);

    <T> List<T> list(String hql, Map<String, Object> params);

    /**
     * 查询全部并排除逻辑删除的数据
     * @param <T>
     * @return
     */
    <T> List<T> findAllWithoutDeleted();

    /**
     * 获取单条数据
     */
    T getByHql(String hql, Object... params);

    /**
     * 获取单条数据
     */
    T getByHql(String hql, Map<String, Object> params);

    /**
     * 列表所有数据获取数据
     */
    <T> List<T> list(String hql, Object... params);

    /**
     * 分页获取数据数据
     */
    PageInfo<T> page(PageInfo pageInfo, String hql, Object... params);

    /**
     * 分页获取数据数据
     */
    PageInfo<T> page(PageInfo pageInfo, String hql, Map<String, Object> params);

    /**
     * 执行修改语句
     */
    int executeUpdate(String hql, Object... params);

    /**
     * 以下方法为对Native Query的支持...
     */
    /**
     * 执行Native Query修改语句
     */
    int executeNativeUpdate(String sql, Object... params);

    /**
     * Native Query获取单条数据
     */
    T getBySql(String sql, Class<T> resultClass, Object... params);

    /**
     * Native Query获取单条数据
     */
    T getBySql(String sql, Class<T> resultClass, Map<String, Object> params);

    /**
     * Native Query列表所有数据获取数据
     */
    <T> List<T> listBySql(String sql, Class<T> resultClass, Object... params);

    /**
     * Native Query列表所有数据获取数据
     */
    <T> List<T> listBySql(String sql, Class<T> resultClass, Map<String, Object> params);

    /**
     * Native Query分页获取数据
     */
    PageInfo<T> pageBySql(PageInfo pageInfo, String sql, Class<T> resultClass, Object... params);

    /**
     * Native Query分页获取数据
     */
    PageInfo<T> pageBySql(PageInfo pageInfo, String sql, Class<T> resultClass, Map<String, Object> params);
}
